:
# NAME:
#	fixhosts.sh - fix a hosts file
#
# DESCRIPTION:
#	This script should be run after updating a hosts file.
#	If $HOST does not appear in /etc/hosts then get the value from
#	hosts.old
#
#	In addition, it makes sure that /etc/hostname.* match
#	/etc/hosts. 
#
# AUTHOR:
#	Simon J. Gerraty <sjg@crufty.net>
#
#	

# RCSid:
#	$Id: fixhosts.sh,v 1.5 2001/08/30 09:55:52 sjg Exp $
#
#	@(#) Copyright (c) 1995 Simon J. Gerraty
#
#	This file is provided in the hope that it will
#	be of use.  There is absolutely NO WARRANTY.
#	Permission to copy, redistribute or otherwise
#	use this file is hereby granted provided that 
#	the above copyright notice and this notice are
#	left intact. 
#      
#	Please send copies of changes and bug-fixes to:
#	sjg@crufty.net
#

Mydir=`dirname $0`
Myname=`basename $0 .sh`
rm -f /tmp/$Myname.*

hosts=$1

ok=`grep "[	 ]$HOST" $hosts`
case "$ok" in
"")
	test -s $hosts.old && grep "[	 ]$HOST" $hosts.old >> $hosts
	;;
esac
# now make sure that name-iface exists
HOST=${HOST:-`uname -n | sed 's/\..*//'`}

OS=${OS:-`uname`}
OSREL=${OS:-`uname -r`}

DEST_HOST=${DEST_HOST:-$HOST}

test "$HOST" = "$DEST_HOST" || exit 0

case "$OS" in
HP-UX)
	netstat -in | 
	awk '/\.[0-9]/ { print "echo", $1, $4, "> /tmp/" Myname "." $1 }' Myname=$Myname - | sh
	;;
*)
	ifconfig -a |
	awk '
/:/ {
	iface=$1
}
/inet/ {
	print iface, $2
}
' | tr -d : | while read iface addr
	do
		echo $iface $addr > /tmp/$Myname.$iface
	done
	;;
esac

# this ensures we process them in the same order as rc.boot
# or whatever... this is important for SunOS.

for f in /tmp/$Myname.*
do
	set -- `cat $f`
	iface=$1
	addr=$2
	
	case $addr in
	127.*)	continue;;
	esac

	# for hosts with multiple interfaces, use host-iface
	# to select correct address from hosts
	name=${HOST}-$iface
	
	case "$OS" in
	SunOS)
		# However, on SunOS the first interface is used to
		# name the box!	 and it gets reall upset if called
		# host-le0 and that is not in the DNS!
		[ "$notfirst" ] || name=$HOST
		;;
	esac
	notfirst=$name
	
	haddr=`grep $name $hosts | sed 's/[	 ].*//' | head -1`
	case "$haddr" in
	$addr)	# they match
		;;
	"")	# missing
		echo "$addr	$name" >> $hosts
		;;
	*)	# missmatch!
		# REVISIT: use addr or haddr?
		echo "$haddr != $addr" >&2
		# if we were using name=$HOST, better try again with
		# name=$HOST-$iface
		if [ "$name" = "$HOST" ]; then
			name=${HOST}-$iface
			haddr=`grep $name $hosts | sed 's/[	 ].*//' | head -1`
			case "$haddr" in
			$addr)	# they match
				;;
			"")	# missing
				echo "$addr	$name" >> $hosts
				;;
			*)	# missmatch!
				echo "help, I'm lost!" >&2
				;;
			esac
		fi
		;;
	esac
	case "$OS" in
	SunOS)
		echo $name > $DESTDIR/etc/hostname.$iface
		;;
	*BSD)
		echo $name inet > $DESTDIR/etc/hostname.$iface
		;;
	esac
done
	
rm -f /tmp/$Myname.*
